
//////////////////////////////////////////////////////////////////////////////
// Includes

#include "fluids/polyhedron.h"
#include "util/assert.h"

using std::vector;


//////////////////////////////////////////////////////////////////////////////
// CalculateCoefficients - Calculate the corner triple products

void Polyhedron::CalculateCoefficients()
{
    for( size_t j = 0; j < m_vertices.size(); ++j )
    {
        vec3 normal1 = m_planeNormals[ m_vertices[j].planes[0] ];
        vec3 normal2 = m_planeNormals[ m_vertices[j].planes[1] ];
        vec3 normal3 = m_planeNormals[ m_vertices[j].planes[2] ];
        m_vertices[j].V = abs( normal1.cross(normal2).dot(normal3) );
    }
}

//////////////////////////////////////////////////////////////////////////////
// BarycentricCoords - Given a point inside this polyhedron, returns the
//                     barycentric weights of each of the vertices.

void Polyhedron::BarycentricCoords( const vec3& p, vector<real>& weights )
{
    const size_t n = m_vertices.size();
    const real epsilon = (real)1.0e-10;

    smart_assert( weights.size() >= n ).fatal();

    real weightSum = (real)0.0;
    for( size_t i = 0; i < n; ++i )
    {
        vec3 offset = m_vertices[i].position - p;

        real n1 = offset.dot( m_planeNormals[ m_vertices[i].planes[0] ] );
        real n2 = offset.dot( m_planeNormals[ m_vertices[i].planes[1] ] );
        real n3 = offset.dot( m_planeNormals[ m_vertices[i].planes[2] ] );

        if( n1 < epsilon )  n1 = epsilon;
        if( n2 < epsilon )  n2 = epsilon;
        if( n3 < epsilon )  n3 = epsilon;

        weights[i] = m_vertices[i].V / ( n1 * n2 * n3 );
        weightSum += weights[i];
    }

    weightSum = (real)1.0 / weightSum;
    for( size_t i = 0; i < n; ++i )
        weights[i] *= weightSum;
}
