#ifndef _SIMPLICIAL_COMPLEX_VERTEX_H_
#define _SIMPLICIAL_COMPLEX_VERTEX_H_


//////////////////////////////////////////////////////////////////////////////
// Includes

#include "util/linear.h"


//////////////////////////////////////////////////////////////////////////////

template<int N>
class SimplicialComplex;


//////////////////////////////////////////////////////////////////////////////
// class Vertex
// A class for doing operations on and querying a 0-simplex of a complex

class Vertex
{
    // friends
    template<typename T>
    friend class SimplexIterator;
    friend class VertexEdgeIterator;
    friend class SimplicialComplex;
    friend class Edge;
    friend class Face;
    friend class Tet;

    friend bool operator==( const Vertex&, const Vertex& );
    friend bool operator!=( const Vertex&, const Vertex& );

public:  // methods
    Vertex();
    ~Vertex();

    operator bool() const;

    size_t Index( void ) const  { return index; }

    vec3 Position  ( void ) const;
    real DualVolume( void ) const;

    VertexEdgeIterator Edges( void ) const;

    bool IsOnBoundary( void ) const;

private:  // methods
    Vertex( const SimplicialComplex<3> *sc, size_t index );

private:  // data
    const SimplicialComplex<3> *sc;
    size_t index;
};


#endif  // _SIMPLICIAL_COMPLEX_VERTEX_H_
