#ifndef _ASSERT_H_
#define _ASSERT_H_

#include "util/filename.h"
#include "util/default_logs.h"
#include <string>
#include <cassert>

#ifdef _MSC_VER
#pragma warning( push )
#pragma warning( disable : 4355 )  // 'this' used in base member initializer list
#endif


class Assert
{
public:
    Assert& m_assert_a;
    Assert& m_assert_b;

    Assert() : m_assert_a(*this), m_assert_b(*this)  { }

    template <typename T> Assert& print_val( const char* str, T val )
    {
        LOG(err) << '\t' << str  << " = " << val;
        return *this;
    }

    Assert& msg( const char* msg )
    {
        LOG(err) << "\t[Msg] " << msg;
        return *this;
    }

    void fatal()
    {
        LOG(err) << "Aborting...";
#ifdef NDEBUG
        exit(1);
#else
        _assert( m_expression.c_str(), m_filename.c_str(), m_line );
#endif
    }

    Assert& fail( const char* expr, unsigned int line, const char* func, const char* file )
    {
        m_expression = expr;
        m_filename = file;
        m_line = line;

        LOG(err) << "Assertion failure in " << func << "(" << line << ")::"
                 << Filename(file).GetFile() << ". Expr is \"" << expr << "\"";

        return *this;
    }

private:
    Assert& operator = ( const Assert& );
    std::string m_expression;
    std::string m_filename;
    unsigned int m_line;
};


#define m_assert_a(x) m_assert_op( x, b )
#define m_assert_b(x) m_assert_op( x, a )

#define m_assert_op( x, next ) m_assert_a.print_val( #x, (x) ).m_assert_ ## next

#ifdef NDEBUG

#define smart_assert( expr ) \
    if( (true) ) ; \
    else Assert().fail( #expr, __LINE__, __FUNCTION__, __FILE__ ).m_assert_a 

#else

#define smart_assert( expr ) \
    if( (expr) ) ; \
    else Assert().fail( #expr, __LINE__, __FUNCTION__, __FILE__ ).m_assert_a 

#endif // NDEBUG


#ifdef _MSC_VER
#pragma warning( pop )
#ifdef NDEBUG
#pragma warning( disable : 4127 )  // conditional expression is constant
#endif
#endif

#endif  // _ASSERT_H_
