#include "util/log/functions.h"
#include <vector>
#include <ctime>
#include <fstream>
#include <iostream>

#ifdef WIN32
#include <windows.h>
#endif

namespace ns_log {

///////////////////////////////////////////////////////////////////////////////////////////////////////
// Log functions

#ifdef WIN32
void write_to_dbg_wnd(const std::string &, const std::string &msg)
{
    OutputDebugString( msg.c_str() );
}
#endif

void write_to_cout(const std::string &, const std::string &msg)
{
    std::cout << msg;
}

write_to_file::write_to_file(const std::string & filename, bool overwrite) : filename(filename)
{
    std::ofstream out( filename.c_str(), std::ios::out | (overwrite ? 0 : std::ios::app) );
    time_t t = time(NULL);
    std::string ts = asctime(localtime(&t));
    ts.erase( ts.length() - 1 );
    out << "----------------Log Begin, " << ts << "----------------\n";
}

void write_to_file::operator()(const std::string &, const std::string &msg) const
{
    std::ofstream out( filename.c_str(), std::ios::out | std::ios::app );
    out << msg;
}


///////////////////////////////////////////////////////////////////////////////////////////////////////
// Modifier functions

void prepend_prefix(const std::string &prefix, std::string & msg)
{
    msg = '[' + prefix + "] " + msg;
}

void append_enter(const std::string &, std::string & msg)
{
    if( msg.empty() ) return;
    if( *msg.rbegin() != '\n' ) msg += '\n';
}

void prepend_time(const std::string &, std::string& msg)
{
    char buffer[10] = {0};
    if( _strtime( buffer ) != 0 )
    {
        buffer[8] = ' ';
        msg = buffer + msg;
    }
}

void appender_array::operator()( const std::string & log_name, const std::string & msg )
{
    for( std::vector<appender_func>::const_iterator iter = m_funcs.begin(); iter != m_funcs.end(); ++iter )
        (*iter)(log_name, msg);
}


}  // namespace ns_log
